/* @file dhcp4_lease.c
 *
 *  Representation of ISC dhcp lease options for libdhcp
 *
 */
/*
 * Copyright (C) 2006  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions of
 * the GNU General Public License v.2.  This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY expressed or
 * implied, including the implied warranties of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.  You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA. Any Red Hat trademarks that are incorporated in the source code or
 * documentation are not subject to the GNU General Public License and may
 * only be used or replicated with the express permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Jason Vas Dias
 *                    David Cantrell
 */
#ifndef _DHCP4_LEASE_H
#define _DHCP4_LEASE_H

#include  <sys/types.h>
#include  <netinet/in.h>
#include  <stdint.h>

/** @addtogroup DHCPv4_lease
 *  @{
 */

/**
 *  DHCP Option Type Codes:
 */
typedef
enum dhcp_option_type__e
{   /* These are the same codes as used by dhcpd / dhclient,
     * and which are written by dhclient with '-x' option to 
     * new option information variables named x._universe_.x:
     */
    /* "atomic" types: */
    DHC_T_IP_ADDRESS        = 'I', /**<  I - IP address */
    DHC_T_HEX_STRING        = 'X', /**<  X - either an ASCII string or binary data.   On output, the string is
                                    *        scanned to see if it's printable ASCII and, if so, output as a
                                    *        quoted string.   If not, it's output as colon-seperated hex.   On
                                    *        input, the option can be specified either as a quoted string or as 
                                    *        a colon-seperated hex list. */
    DHC_T_DOMAIN_NAME       = 'd', /**<  d - Domain name (i.e., FOO or FOO.BAR). */
    DHC_T_INT32             = 'l', /**<  l - 32-bit signed integer */
    DHC_T_UINT32            = 'L', /**<  L - 32-bit unsigned integer */
    DHC_T_INT16             = 's', /**<  s - 16-bit signed integer */
    DHC_T_UINT16            = 'S', /**<  S - 16-bit unsigned integer */
    DHC_T_CHAR              = 'b', /**<  b - 8-bit signed integer */
    DHC_T_UCHAR             = 'B', /**<  B - 8-bit unsigned integer */
    DHC_T_TEXT              = 't', /**<  t - ASCII text */
    DHC_T_BOOL              = 'f', /**<  f - flag (true or false) */
    DHC_T_IMPLICIT_BOOL     = 'F', /**<  F - implicit flag - the presence of the option indicates that the */

    /* compound types: */
    DHC_T_ARRAY             = 'A', /**<  A - array of whatever precedes (e.g., IA means array of IP addresses) */
    DHC_T_LIST              = 'a', /**<  a - array of the preceding character (e.g., IIa means two or more IP
                                     *       addresses) */
    DHC_T_ENUMERATION       = 'N', /**<  N - enumeration.   N is followed by a text string containing 
                                    *        the name of the set of enumeration values to parse or emit,
                                    *        followed by a '.'.   The width of the data is specified in the
                                    *        named enumeration.   Named enumerations are tracked in parse.c. */
    /* these are types internal to DHCP and can  safely be ignored .
     * ( they are not emitted by dhclient in environment -
     *   except "vendor-option-space", which we deal with by use of
     *   '._universe_.' information.
     * )
     */
    DHC_T_OPTION_SPACE      = 'U', /**<  U - name of an option space (universe) */
    DHC_T_OPTIONAL          = 'o', /**<  o - the preceding value is optional. */
    DHC_T_ENCAPSULATION     = 'E', /**<  E - encapsulation, string or colon-seperated hex list (the latter 
                                    *        two for parsing).   E is followed by a text string containing
                                    *        the name of the option space to encapsulate, followed by a '.'.
                                    *        If the E is immediately followed by '.', the applicable vendor
                                    *        option space is used if one is defined. */
    DHC_T_ENCAPSULATION_NO_DIRECTIVE= 'e',/**<  e - If an encapsulation directive is not the first thing in the string,
                                    *        the option scanner requires an efficient way to find the encapsulation.
                                    *        This is done by placing a 'e' at the beginning of the option.   The
                                    *        'e' has no other purpose, and is not required if 'E' is the first
                                    *        thing in the option. */
    DHC_T_NONE               = '\0'
}   DHCPv4_Option_Type;

/**
 * DHCPv4_Option_Structure_Type - used internally by the DHCPv4_lease parser.
 */
typedef
enum dhcpv4_option_structure_type_e
{
    DHCO_SCALAR,                   /**< SCALAR option */
    DHCO_ARRAY,                    /**< ARRAY option */
    DHCO_STRUCT                    /**< STRUCT option */
}   DHCPv4_Option_Structure_Type;

/**
 *  The maximum length in bytes of a DHCP option:
 */
#define DHCPV4_MAX_OPTION_LENGTH  1236

/**
 * The DHCP Option structure.
 *
 * Structured options (options with more than one member)
 * are laid out as C structures in 'opt->value', with a
 * list of pointers to each member following the C structure.
 * opt->length will be :
 * sizeof(structure) + (n_members * sizeof(void*)).
 * ie. sizeof(structure) 
 *     == opt->length -(n_members * sizeof(void*)).
 *
 * Array options have exactly opt->n_members elements of opt->size,
 * and are packed as a c array.
 *
 * For scalars, n_members == 1.
 */

typedef struct dhcp4_option_s
{   /* Generic option structure */
    struct dhcp4_lease_s *lease;      /**< the lease this option is associated with */
    char *                name;       /**< dhcp option name, defined in dhcp/common/tables.c */
    char *                format;     /**< dhcp format string of DHCPv4_Option_Type(s) */
    char *                universe;   /**< universe name */
    uint8_t               unicode;    /**< universe code */
    uint8_t               code;       /**< 0 - 255 */
    uint8_t               form;       /**< DHCPv4_Option_Structure_Type */
    uint8_t               size;       /**< If struct / array, size of element */
    uint8_t               udefined;   /**< 1 if user defined option */
    uint8_t               redefined;  /**< 1 if user redefined option*/
    uint16_t              n_members;  /**< > 1 if structure */ 
    uint16_t              n_elements; /**< > 1 if array */   
    uint16_t              length;     /**< size of value in bytes */
    uint8_t               **members;  /**< points to array of pointers to members at end of value */
    uint8_t               value[1];   /**< option value(s) c-structure ; must be aligned for integer */
} DHCPv4_option;


/**
 * The DHCP Option Callback function type
 */
typedef  void (*DHCPv4_option_handler)(DHCPv4_option *, void* );

/**
 * The DHCP lease structure
 */
typedef struct dhcp4_lease_s
{
    struct in_addr address;           /**< client IP address being leased    */ 
    struct in_addr requested_address; /**< address we requested, (if any)    */
    struct in_addr server_address;    /**< boot server IP address (if any)   */
    char *filename;                   /**< boot filename                     */
    char *server_name;                /**< boot server name                  */
    uint8_t is_static;                /**< 1: was from config, not pool      */
    uint8_t is_bootp;                 /**< 1: is a bootp lease               */
    uint16_t if_index;                /**< lease on interface with this index*/
    char *if_name;                    /**< lease on interface with this name */
    time_t  requested,                /**< time that lease was requested     */
            expiry, renewal, rebind;  /**< times in seconds since epoch      */
    void *options;                    /**< glibc tsearch(3) btree of options */
    void *options_by_name;            /**< glibc tsearch(3) btree of options */
    DHCPv4_option_handler handler;    /**< handler for this lease            */
    void *handler_arg;                /**< argument to pass to handler       */
} DHCPv4_lease;

struct client_state;                  /**< include "dhcpd.h" to get this     */

extern DHCPv4_lease *dhcpv4_lease( struct client_state * ); /**<
     * creates a DHCPv4_lease from an ISC DHCP 'struct client_state'.
     */

extern void dhcpv4_lease_free( DHCPv4_lease * ); /**<
     * frees all resources associated with lease 
     */

extern void dhcpv4_process_options ( DHCPv4_lease *, DHCPv4_option_handler, void *); /**<
     * calls handler with each option in lease and the void* arg 
     */

extern DHCPv4_option *dhcpv4_get_option_by_code( DHCPv4_lease *, uint8_t universe, uint8_t code); /**<
     * returns a DHCP lease option by universe and option code, or NULL
     */ 

extern DHCPv4_option *dhcpv4_get_option_by_name( DHCPv4_lease *, char *);  /**<
     * returns a DHCP lease option by name, or NULL
     */ 

extern int dhcpv4_pack_lease ( DHCPv4_lease*, uint8_t *buf, uint32_t len); /**<
     * packs a DHCPv4_lease in a buffer of length length, suitable for 
     * IPC / mmap'd file storage.
     */

extern DHCPv4_lease *dhcpv4_unpack_lease (uint8_t *buf); /**<
     * unpacks a DHCPv4_lease from a buffer packed by dhcpv4_pack_lease.
     */

/** DHCP Code Spaces:
 *
 * Option Codes have unique values in one of these "DHCP_Universe" 
 * code spaces:
 */

/**  DHCP can define no more than 256 options, 0-255, in a "universe" */
#define DHCP_OPTION_MAX   256

/**  DHCP currently supports no more than 10 universes (a hardcoded constant in tables.c).
 *  It defines 4 of its own and supports up to 5 user defined option spaces.
 */
#define DHCP_UNIVERSE_MAX 10

/**  "Universe" codes defined by ISC DHCP: */
typedef 
enum dhcp_u_e
{
    DHC_DHCP_Universe=0,           /**< Proper DHCP Options as defined in RFC 1533 */
    DHC_RAI_Universe=1,            /**< Relay Agent Information sub-option encapsulation */
    DHC_NWIP_Universe=2,           /**< NetWare/IP sub-option encapsulation */
    DHC_FQDN_Universe=3,           /**< Fully Qualified Domain Name sub-option encapsulation */
    DHC_DEFINED_UNIVERSE_N=4
} DHCP_Universe;

/**  DHCP Universe  Option Codes - 
 *   Codes of options in the DHCP universe.
 *   enum value names map to option names converted to lowercase, 
 *   with '_' replaced by '-'.
 */
typedef
enum dhcp_oc_e
{
    DHCO_PAD				=	0, /**< Unused */
    DHCO_SUBNET_MASK			=	1, /**< subnet-mask */
    DHCO_TIME_OFFSET			=	2, /**< time-offset */
    DHCO_ROUTERS			=	3, /**< routers */
    DHCO_TIME_SERVERS			=	4, /**< time-servers */
    DHCO_NAME_SERVERS			=	5, /**< name-servers */
    DHCO_DOMAIN_NAME_SERVERS		=	6, /**< domain-name-servers */
    DHCO_LOG_SERVERS			=	7, /**< log-servers */
    DHCO_COOKIE_SERVERS			=	8, /**< cookie-servers */
    DHCO_LPR_SERVERS			=	9, /**< lpr-servers */
    DHCO_IMPRESS_SERVERS		=	10, /**< impress-servers */
    DHCO_RESOURCE_LOCATION_SERVERS	=	11, /**< resource-location-servers */
    DHCO_HOST_NAME			=	12, /**< host-name */
    DHCO_BOOT_SIZE			=	13, /**< boot-size */
    DHCO_MERIT_DUMP			=	14, /**< merit-dump */
    DHCO_DOMAIN_NAME			=	15, /**< domain-name */
    DHCO_SWAP_SERVER			=	16, /**< swap-server */
    DHCO_ROOT_PATH			=	17, /**< root-path */
    DHCO_EXTENSIONS_PATH		=	18, /**< extensions-path */
    DHCO_IP_FORWARDING			=	19, /**< ip-forwarding */
    DHCO_NON_LOCAL_SOURCE_ROUTING	=	20, /**< non-local-source-routing */
    DHCO_POLICY_FILTER			=	21, /**< policy-filter */
    DHCO_MAX_DGRAM_REASSEMBLY		=	22, /**< max-dgram-reassembly */
    DHCO_DEFAULT_IP_TTL			=	23, /**< default-ip-ttl */
    DHCO_PATH_MTU_AGING_TIMEOUT		=	24, /**< path-mtu-aging-timeout */
    DHCO_PATH_MTU_PLATEAU_TABLE		=	25, /**< path-mtu-plateau-table */
    DHCO_INTERFACE_MTU			=	26, /**< interface-mtu */
    DHCO_ALL_SUBNETS_LOCAL		=	27, /**< all-subnets-local */
    DHCO_BROADCAST_ADDRESS		=	28, /**< broadcast-address */
    DHCO_PERFORM_MASK_DISCOVERY		=	29, /**< perform-mask-discovery */
    DHCO_MASK_SUPPLIER			=	30, /**< mask-supplier */
    DHCO_ROUTER_DISCOVERY		=	31, /**< router-discovery */
    DHCO_ROUTER_SOLICITATION_ADDRESS	=	32, /**< router-solicitation-address */
    DHCO_STATIC_ROUTES			=	33, /**< static-routes */
    DHCO_TRAILER_ENCAPSULATION		=	34, /**< trailer-encapsulation */
    DHCO_ARP_CACHE_TIMEOUT		=	35, /**< arp-cache-timeout */
    DHCO_IEEE802_3_ENCAPSULATION	=	36, /**< ieee802-3-encapsulation */
    DHCO_DEFAULT_TCP_TTL		=	37, /**< default-tcp-ttl */
    DHCO_TCP_KEEPALIVE_INTERVAL		=	38, /**< tcp-keepalive-interval */
    DHCO_TCP_KEEPALIVE_GARBAGE		=	39, /**< tcp-keepalive-garbage */
    DHCO_NIS_DOMAIN			=	40, /**< nis-domain */
    DHCO_NIS_SERVERS			=	41, /**< nis-servers */
    DHCO_NTP_SERVERS			=	42, /**< ntp-servers */
    DHCO_VENDOR_ENCAPSULATED_OPTIONS	=	43, /**< vendor-encapsulated-options */
    DHCO_NETBIOS_NAME_SERVERS		=	44, /**< netbios-name-servers */
    DHCO_NETBIOS_DD_SERVER		=	45, /**< netbios-dd-server */
    DHCO_NETBIOS_NODE_TYPE		=	46, /**< netbios-node-type */
    DHCO_NETBIOS_SCOPE			=	47, /**< netbios-scope */
    DHCO_FONT_SERVERS			=	48, /**< font-servers */
    DHCO_X_DISPLAY_MANAGER		=	49, /**< x-display-manager */
    DHCO_DHCP_REQUESTED_ADDRESS		=	50, /**< dhcp-requested-address */
    DHCO_DHCP_LEASE_TIME		=	51, /**< dhcp-lease-time */
    DHCO_DHCP_OPTION_OVERLOAD		=	52, /**< dhcp-option-overload */
    DHCO_DHCP_MESSAGE_TYPE		=	53, /**< dhcp-message-type */
    DHCO_DHCP_SERVER_IDENTIFIER		=	54, /**< dhcp-server-identifier */
    DHCO_DHCP_PARAMETER_REQUEST_LIST	=	55, /**< dhcp-parameter-request-list */
    DHCO_DHCP_MESSAGE			=	56, /**< dhcp-message */
    DHCO_DHCP_MAX_MESSAGE_SIZE		=	57, /**< dhcp-max-message-size */
    DHCO_DHCP_RENEWAL_TIME		=	58, /**< dhcp-renewal-time */
    DHCO_DHCP_REBINDING_TIME		=	59, /**< dhcp-rebinding-time */
    DHCO_VENDOR_CLASS_IDENTIFIER	=	60, /**< vendor-class-identifier */
    DHCO_DHCP_CLIENT_IDENTIFIER		=	61, /**< dhcp-client-identifier */
    DHCO_NWIP_DOMAIN_NAME		=	62, /**< nwip-domain-name */
    DHCO_NWIP_SUBOPTIONS		=	63, /**< nwip-suboptions */
    DHCO_NISPLUS_DOMAIN                 =       64, /**< nisplus-domain */
    DHCO_NISPLUS_SERVERS                =       65, /**< nisplus-servers */
    DHCO_TFTP_SERVER_NAME               =       66, /**< tftp-server-name */
    DHCO_BOOTFILE_NAME                  =       67, /**< bootfile-name */
    DHCO_MOBILE_IP_HOME_AGENT           =       68, /**< mobile-ip-home-agent */
    DHCO_SMTP_SERVER                    =       69, /**< smtp-server */
    DHCO_POP_SERVER                     =       70, /**< pop-server */
    DHCO_NNTP_SERVER                    =       71, /**< nntp-server */
    DHCO_WWW_SERVER                     =       72, /**< www-server */
    DHCO_FINGER_SERVER                  =       73, /**< finger-server */
    DHCO_IRC_SERVER                     =       74, /**< irc-server */
    DHCO_STREETTALK_SERVER              =       75, /**< streettalk-server */
    DHCO_STREETTALK_DIRECTORY_ASSISTANCE_SERVER=76, /**< streettalk-directory-assistance-server */
    DHCO_USER_CLASS			=	77, /**< user-class */
    DHCO_SLP_DIRECTORY_AGENT            =       78, /**< slp-directory-server */
    DHCO_SLP_SERVICE_SCOPE              =       79, /**< slp-service-scope */
    DHCO_FQDN				=	81, /**< fqdn */
    DHCO_DHCP_AGENT_OPTIONS		=	82, /**< dhcp-agent-options */
    DHCO_NDS_SERVERS                    =       85, /**< nds-servers */
    DHCO_NDS_TREE_NAME                  =       86, /**< nds-tree-name */
    DHCO_NDS_CONTEXT                    =       87, /**< nds-context */
    DHCO_UAP_SERVERS                    =       98, /**< uap-servers */
    DHCO_SUBNET_SELECTION		=	118, /**< subnet-selection */
    DHCO_AUTHENTICATE			=	210, /**< authenticate */
    DHCO_END                            =       210, /**< */
    DHCO_N                              =       211 /**< */
}   DHCP_Option_Code;

/** DHCP Relay Agent Information Sub-Options:
 */
typedef
enum dho_rai_c_e
{
    DHCO_RAI_PAD                         =       0, /**< */
    DHCO_RAI_CIRCUIT_ID	                 =       1, /**< rai.circuit-id */
    DHCO_RAI_REMOTE_ID	                 =       2, /**< rai.remote-id */
    DHCO_RAI_AGENT_ID	                 =       3, /**< rai.agent-id */
    DHCO_RAI_END                         =       3, /**< */
    DHCO_RAI_N                           =       4  /**< */
} DHCP_RAI_Options;

/** DHCP NetWare/IP (nwip) Sub-Options:
 */
typedef
enum dho_nwip_c_e
{
    DHCO_NWIP_PAD                        =       0, /**< */
    DHCO_NWIP_ILL1                       =       1, /**< */
    DHCO_NWIP_ILL2                       =       2, /**< */
    DHCO_NWIP_ILL3                       =       3, /**< */
    DHCO_NWIP_ILL4                       =       4, /**< */
    DHCO_NWIP_NSQ_BROADCAST              =       5, /**< nwip.nsq-broadcast */
    DHCO_NWIP_PREFERRED_DSS              =       6, /**< nwip.preferred-dss */
    DHCO_NWIP_NEAREST_NWIP_SERVER        =       7, /**< nwip.nearest-nwip-server */
    DHCO_NWIP_AUTORETRIES                =       8, /**< nwip.autoretries */
    DHCO_NWIP_AUTORETRY_SECS             =       9, /**< nwip.autoretry-secs */
    DHCO_NWIP_1_1                        =      10, /**< nwip.1-1 */
    DHCO_NWIP_PRIMARY_DSS                =      11, /**< nwip.primary-dss */
    DHCO_NWIP_END                        =      11, /**< */
    DHCO_NWIP_N                          =      12  /**< */
} DHCP_NWIP_Options;

/** DHCP Fully Qualified Domain Name (fqdn) Sub-Options:
 */
typedef
enum dho_fqdn_c_e
{
    DHCO_FQDN_PAD                        =       0, /**< */
    DHCO_FQDN_NO_CLIENT_UPDATE		 =       1, /**< fqdn.no-client-update */
    DHCO_FQDN_SERVER_UPDATE		 =       2, /**< fqdn.server-update */
    DHCO_FQDN_ENCODED			 =       3, /**< fqdn.encoded */
    DHCO_FQDN_RCODE1			 =       4, /**< fqdn.rcode1 */
    DHCO_FQDN_RCODE2			 =       5, /**< fqdn.rcode2 */
    DHCO_FQDN_HOSTNAME			 =       6, /**< fqdn.hostname */
    DHCO_FQDN_DOMAINNAME		 =       7, /**< fqdn.domain-name */
    DHCO_FQDN_FQDN			 =       8, /**< fqdn.fqdn */
    DHCO_FQDN_END                        =       8, /**< */
    DHCO_FQDN_N                          =	 9  /**< */
} DHCP_FQDN_Options;

/** NOTE: programs using DHCP must be able to handle (or ignore)
 * user defined options in new universes with user defined formats,
 * and handle existing defined options that have been redefined by
 * the user (ISC dhcp allows this!).
 */
/**@}*/
#endif
