/* @file dhcp4_nic.h
 *
 * Network Interface Configurator for the ISC DHCP IPv4 client library API.
 *
 */
/*
 * Copyright (C) 2006  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions of
 * the GNU General Public License v.2.  This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY expressed or
 * implied, including the implied warranties of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.  You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA. Any Red Hat trademarks that are incorporated in the source code or
 * documentation are not subject to the GNU General Public License and may
 * only be used or replicated with the express permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Jason Vas Dias
 *                    David Cantrell
 */
#include <libdhcp.h>
#include <dhcp4_lease.h>
#include <nic.h>
#include <stdarg.h>

/**
 * @addtogroup  DHCPv4
 * @{
 */
typedef struct dhcpv4_control_s DHCPv4_control; /**<
 * The DHCPv4 client control structure. This has more
 * fields than the plain LIBDHCP_Control structure,
 * and must be obtained by calling dhcpv4_control.
 */

/**
 * dhcpv4_control(): returns a new DHCPv4_control structure.
 */
extern
DHCPv4_control*
dhcpv4_control
(
    NLH_t                  nic_handle,     /**< nic library handle - @ref NICH */
    char                  *eth_if_name,    /**< ethernet interface name */
    LIBDHCP_Capability     dhc_cap,        /**< DHCP capability - @see ::LIBDHCP_Capability */
    time_t                 timeout,        /**< timeout - @see ::LIBDHCP_Control::timeout */
    LIBDHCP_Error_Handler  error_handler,  /**< error handler - @see ::LIBDHCP_Control::error_handler */
    uint8_t                log_level,      /**< log_level - @see ::LIBDHCP_Control::log_level */
    ... /**< extra dhclient args - see man 8 dhclient -
         * last arg MUST be 0.
         */
);

/**
 * dhcpv4_control_va - dhcpv4_control() that can be called with a va_list.
 */
extern
DHCPv4_control*
dhcpv4_control_va
(
    NLH_t                  nic_handle,
    char                  *eth_if_name,
    LIBDHCP_Capability     dhc_cap, 
    time_t                 timeout,    
    LIBDHCP_Error_Handler  error_handler,
    uint8_t                log_level,
    va_list                dhclient_va
);

/**
 * dhcpv4_control_free( ctl ) : frees all resources associated with ctl.
 */
extern
void dhcpv4_control_free( DHCPv4_control* );

/**
 * DHCPv4_nic: structure representing the network interface configuration
 * parameters associated with a DHCPv4_lease.
 */
typedef
struct dhcp4_nic_s
{
    NLH_t          nh;           /**< NIC library handle */
    NIC_t          nic;          /**< Network Interface structure */
    IPaddr_list_t  address_list; /**< list of lease addresses */
    IProute_list_t route_list;   /**< list of lease routes */
    IPaddr_list_t  dns_list;     /**< list of resolver DNS servers */
    char          *search_list;  /**< DNS resolver search list */
    char          *host_name;    /**< lease host-name option */
    DHCPv4_lease  *lease;        /**< the DHCPv4_lease structure */
} DHCPv4_nic;

extern
DHCPv4_nic *do_dhcpv4( DHCPv4_control * );

extern
NIC_Res_t dhcpv4_nic(NLH_t, DHCPv4_nic * );

extern
uint32_t dhcpv4_mtu_option( DHCPv4_control * );

extern
int dhcp4_process_lease(DHCPv4_control *);

extern 
DHCPv4_nic *dhcp4_set_lease(DHCPv4_control *, DHCPv4_lease *);

extern
void dhcp4_nic_option_handler
(   DHCPv4_option *option, 
    void *arg                 /* must be a DHCPv4_control */
);

extern
int dhcp4_nic_callback
(   LIBDHCP_Control *control, /* must be a DHCPv4_control */
    DHCP_State state, 
    void *arg
);

/**@}*/
