/* @file dhcp_nic.h
 *
 * Network Interface Configurator for BOTH the ISC DHCP IPv4 client library
 * and the DHCPv6 IPv6 client, API.
 *
 */
/*
 * Copyright (C) 2006  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions of
 * the GNU General Public License v.2.  This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY expressed or
 * implied, including the implied warranties of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.  You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA. Any Red Hat trademarks that are incorporated in the source code or
 * documentation are not subject to the GNU General Public License and may
 * only be used or replicated with the express permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Jason Vas Dias
 *                    David Cantrell
 */

#include <libdhcp.h>
#include <nic.h>
#include <dhcp4_nic.h>
#include <dhcp4_lease.h>
#include <dhcp6_nic.h>
#include <dhcp6_lease.h>
#include <malloc.h>

/**
 * @addtogroup DHCP
 * @{
 */

/**
 * The DHCP_Preference enumeration.
 *
 * Controls which clients are to be run, and which configuration actions 
 * are to be enabled on receipt of a lease.
 */
typedef
enum 
{
    DHCPv4_DISABLE  = 1,  /**< disables the DHCPv4 client */
    DHCPv6_DISABLE  = 2,  /**< disabled the DHCPv6 client */
    IPv6_PREFERENCE = 4,  /**< If neither DHCPv4_DISABLE nor DHCPv6_DISABLE are enabled,
                           * setting this will run the DHCPv6 client first, and configure 
			   * IPv6 addresses / routes / DNS first */ 
    DHCPv4_DISABLE_ADDRESSES = 8,  /**< Don't configure DHCPv4 addresses */
    DHCPv4_DISABLE_ROUTES    = 16, /**< Don't configure DHCPv4 routes */
    DHCPv4_DISABLE_RESOLVER  = 32, /**< Don't configure DHCPv4 resolv.conf entries */
    DHCPv6_DISABLE_ADDRESSES = 64, /**< Don't configure DHCPv6 address (ie. use radvd) */
    DHCPv6_DISABLE_RESOLVER  = 128,/**< Don't configure DHCPv6 resolv.conf entries */
    DHCPv4_DISABLE_HOSTNAME_SET=256,/**< Don't set hostname if DHCPv4 host-name option sent */ 
    DHCP_ACCEPT_FIRST_LEASE  = 512 /**< If timeout == 0, and both clients are enabled,
                                    * v4 and v6 clients are run in separate  processes; 
                                    * if this preference is set, then the first client
				    * to get a lease will cause that lease to be accepted
				    * and the other client process to be terminated. */
}   DHCP_Preference;

/**
 *  DHCP_config: The DHCP network interface configuration structure.
 *
 *  This structure encapsulates the network configuration information returned
 *  in a DHCP lease, and the lease itself.
 */
typedef
struct dhcpc_nic_s
{
    NLH_t          nh;               /**< nic library handle */
    NIC_t          nic;              /**< network interface  */
    IPaddr_list_t  address_list;     /**< list of addresses assigned to the nic by the lease */
    IProute_list_t route_list;       /**< list of routes assigned to the nic by the lease */
    IPaddr_list_t  dns_list;         /**< list of DNS servers assigned by the lease */
    char          *search_list;      /**< DNS search list assigned by the lease */ 
    char          *host_name;        /**< host name assigned by the lease */
    union 
    { 
	DHCPv4_lease  *dhcp4_lease;  /**< DHCPv4 lease - see @ref DHCPv4  */
	DHCPv6_lease  *dhcp6_lease;  /**< DHCPv6 lease - see @ref DHCPv6  */
    } lease;
} DHCP_config;

/**
 * DHCP_nic: The DHCP network interface control structure.
 *
 * This structure encapsulates the control and configuration
 * of both DHCP clients.
 */
typedef
struct dhcp_nic
{
    DHCPv4_control  *dhc4ctl;        /**< DHCPv4 control - see @ref DHCPv4 */
    LIBDHCP_Control *dhc6ctl;        /**< DHCPv6 control - see @ref DHCPv6 */
    NLH_t            nh;             /**< nic library handle - see @ref NIC */
    DHCP_config     *dhcpv4;         /**< DHCPv4 configuration - see @ref DHCPv4 */
    DHCP_config     *dhcpv6;         /**< DHCPv6 configuration - see @ref DHCPv6 */
    DHCP_Preference preference;      /**< DHCP_Preference - @see ::DHCP_Preference */
} DHCP_nic;

/**
 * dhcp_nic() : function which invokes the DHCP clients and returns a network interface configuration.
 *
 */
extern 
DHCP_nic *dhcp_nic
( 
    NLH_t nh,                               /**< nic library handle - see  @ref NIC */
    DHCP_Preference preference,             /**< DHCP_Preference - @see ::DHCP_Preference */
    char                  *eth_if_name,     /**< network interface name - eg. 'eth0' */
    LIBDHCP_Capability     dhc_cap,         /**< DHCP_Capability - @see ::DHCP_Capability */
    time_t                 timeout,         /**< timeout - @see ::LIBDHCP_Control::timeout */
    LIBDHCP_Error_Handler  error_handler,   /**< error handler - @see LIBDHCP_Control::error_handler */
    uint8_t                log_level,       /**< log level - @see LIBDHCP_Control::log_level */
    ... /**< extra DHCPv4 dhclient arguments;
	 * last arg MUST be 0 .
	 */
);

/**
 * dhcp_nic_configure () : function to apply the DHCP configuration to the network interface .
 */
extern 
NIC_Res_t
dhcp_nic_configure
(
    DHCP_nic *dhcp_nic  /**< @see ::DHCP_nic */
);

/**
 *  function dhcp_nic_free() : frees all resources associated with the DHCP_nic structure  
 */
extern
void dhcp_nic_free(DHCP_nic *);

/**
 * do_dhcp(): function to run the clients and apply the DHCP configuration to the network interface
 */
extern 
NIC_Res_t do_dhcp
( 
    DHCP_Preference preference,           /**< DHCP_Preference - @see ::DHCP_Preference */
    char                  *eth_if_name,   /**< interface name */
    LIBDHCP_Capability     dhc_cap,       /**< DHCP_Capability - @see ::DHCP_Capability */
    time_t                 timeout,       /**< timeout - @see ::LIBDHCP_Control::timeout */
    LIBDHCP_Error_Handler  error_handler, /**< error_handler @see ::LIBDHCP_Control::error_handler */
    uint8_t                log_level,     /**< log_level - @see ::LIBDHCP_Control::log_level */
    ... /**< extra DHCPv4 dhclient arguments;
	 * last arg MUST be 0 .
	 */
);

/**@}*/

