/** @file ip_addr.h
 *
 * Generic IP address representation and conversion utilities.
 *  
 */
/*
 * Copyright (C) 2006  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions of
 * the GNU General Public License v.2.  This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY expressed or
 * implied, including the implied warranties of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.  You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA. Any Red Hat trademarks that are incorporated in the source code or
 * documentation are not subject to the GNU General Public License and may
 * only be used or replicated with the express permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Jason Vas Dias
 *                    David Cantrell
 */
#ifndef IP_ADDR_H
#define IP_ADDR_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdint.h>
#ifndef AF_LLC
#define AF_LLC 26
#endif
#ifndef __LLC_SOCK_SIZE__
#define __LLC_SOCK_SIZE__ 16
#endif

/**
 * @addtogroup IP
 * @{
 */
/**
 * IP addresses structure big enough for an IPv6 address by default 
 * (minimum size == sizeof(struct sockaddr_in6)).
 * Compatible with all of struct sockaddr, struct sockaddr_in, and
 * struct sockaddr_in6.
 * If used to represent an IPv4 or IPv6 address,
 * (&(s->sin_addr) ==  &(s->sin6_addr) == &(s->sa_pad)) .
 * If used to represent an AF_LLC address (ethernet), 
 * &(((ip_addr_t*)s)->sa_data) == &(((struct sockaddr*)s)->sa_data).
 *
 */
typedef struct ip_sockaddr_s
{
    uint16_t sa_family; /* __SA_COMMON */
    uint8_t  sa_data[ sizeof(struct sockaddr_in6) - sizeof(uint16_t) ];
} __attribute__((aligned(__alignof__(__ss_aligntype)))) ip_addr_t;

/**
 * IP_ADDR_SIZE macro: returns the actual size of the ip_addr_t structure pointed to by 'ip'.
 */
#define IP_ADDR_SIZE( ip ) (  ((ip)->sa_family == AF_INET) ? sizeof(struct sockaddr_in) \
			      :(((ip)->sa_family == AF_INET6) ? sizeof(struct sockaddr_in6) \
				:(((ip)->sa_family == AF_LLC) ? __LLC_SOCK_SIZE__ : 0) \
			       ) \
	                   ) 
	
/* Yes, we should really handle AF_UNIX, X.25, ATM, IRDA, ... etc. sockaddr types also,
 * but libnl doesn't, and we don't want to include kernel headers, so we restrict
 * ourselves to the types that libnl handles.
 */

/**
 * IP_ADDR_IN macro: returns the address of the AF_INET IPv4 address field within the 
 * ip_addr_t structure pointed to by 'ip'.
 */
#define IP_ADDR_IN( ip )  (&((((struct sockaddr_in*)(ip))->sin_addr)))

/**
 * IP_ADDR_IN6 macro: returns the address of the AF_INET6 IPv6 address field within the 
 * ip_addr_t structure pointed to by 'ip'.
 */
#define IP_ADDR_IN6( ip ) (&((((struct sockaddr_in6*)(ip))->sin6_addr)))

/**
 * IP_ADDR_LLC macro: returns the address of the AF_LLC Link Layer (ethernet) address 
 * field within the ip_addr_t structure pointed to by 'ip'.
 */
#define IP_ADDR_LLC( ip ) (&((ip)->sa_data[(2*sizeof(__ss_aligntype))-sizeof(sa_family_t)]))

/**
 * IP_ADDR macro: returns the address of the IP address 
 * field within the ip_addr_t structure pointed to by 'ip'.
 */
#define IP_ADDR( ip ) (  ((ip)->sa_family == AF_INET)    ? (void*) IP_ADDR_IN(ip) \
			 :(((ip)->sa_family == AF_INET6) ? (void*) IP_ADDR_IN6(ip) \
			   :(((ip)->sa_family == AF_LLC) ? (void*) IP_ADDR_LLC(ip) : NULL ) \
			  ) \
	              )

/**
 * IP_ADDR_AF macro: returns the address of the IP address 
 * of family 'af' field within the ip_addr_t structure pointed to by 'ip'.
 */
#define IP_ADDR_AF( ip, af) (  ((af) == AF_INET)    ? (void*) IP_ADDR_IN(ip) \
			       :(((af) == AF_INET6) ? (void*) IP_ADDR_IN6(ip) \
				 :(((af) == AF_LLC) ? (void*) IP_ADDR_LLC(ip) : NULL ) \
			        ) \
	                    )

/**
 * IP_STRLEN macro: returns the maximum string length based on the address
 * family of the ip_addr_t structure. Used in inet_ntop() calls.
 */
#define IP_STRLEN( ip ) \
    ( ((ip)->sa_family == AF_INET) ? INET_ADDRSTRLEN : \
      ((ip)->sa_family == AF_INET6) ? INET6_ADDRSTRLEN : 0 )

/**
 * ip_addr_v4(uint32_t int_ip): returns the ip_addr_t struct of the 32-bit AF_INET address
 * int_ip, converting int_ip to network byte order.
 */
extern
ip_addr_t ip_addr_v4( register uint32_t );

/**
 * ip_addr_v4(ip): returns the the 32 bit integer representation of the AF_INET address
 * represented by ip_addr_t IP, in host byte order.
 */
extern
uint32_t ip_v4_addr( register ip_addr_t* );

/**
 * ip_addr_in( in_addr ): returns the ip_addr_t representation of the struct in_addr
 * AF_INET address argument.
 */
extern
ip_addr_t ip_addr_in( register struct in_addr * );

/**
 * ip_in_addr(ip): returns the struct in_addr AF_INET address represented by the 
 * ip_addr_t pointed to by ip.
 */
extern
struct in_addr ip_in_addr( register ip_addr_t * );

/**
 * ip_addr_sin( sin ): returns the ip_addr_t representation of the struct sockaddr_in
 * pointer sin.
 */
extern 
ip_addr_t ip_addr_sin( register struct sockaddr_in * );

/**
 * ip_sin_addr( ip ): returns the struct sockaddr_in representation of the ip_addr_t
 * pointed to by ip.
 */
extern
struct sockaddr_in ip_sin_addr( register ip_addr_t * );

/**
 * ip_addr_in6( in6 ): returns the ip_addr_t representation of the struct in6_addr
 * pointed to by in6 .
 */
extern
ip_addr_t ip_addr_in6( register struct in6_addr * );

/**
 * ip_in6_addr( ip ): returns the struct in6_addr representation of the ip_addr_t
 * pointed to by ip.
 */
extern
struct in6_addr ip_in6_addr( register ip_addr_t *);


/**
 * ip_addr_sin6( sin6 ): returns the ip_addr_t representation of the struct sockaddr_in6
 * pointed to by sin6.
 */
extern
ip_addr_t ip_addr_sin6(register struct sockaddr_in6 * );

/**
 * ip_sin6_addr( ip ): returns the sockaddr_in6 representation of the ip_addr_t pointed to
 * by ip.
 */
extern
struct sockaddr_in6 ip_sin6_addr( register  ip_addr_t * );


/**
 * struct in6_bytes_s: representation of an IPv6 address as an array of 16 bytes: 
 */
typedef 
struct in6_bytes_s
{ uint8_t a[16];
} in6_bytes_t;

/**
 * ip_addr_in6bytes( in6bytes ): returns the ip_addr_t representation of the
 * in6_bytes_t pointed to by in6_bytes
 */
extern
ip_addr_t ip_addr_in6bytes( register in6_bytes_t * );

/**
 * ip_in6bytes_addr( ip ): returns the in6_bytes_t representation of the 
 * ip_addr_t pointed to by ip.
 */
extern
in6_bytes_t ip_in6bytes_addr( register ip_addr_t * );

/**
 * ip_addr_text( str ): converts string str to an IP address represented
 * by the returned ip_addr_t, if possible. If not possible, a null IP address
 * (with sa_family == 0) is returned.
 */
extern
ip_addr_t ip_addr_text( register const char * ); 

/**
 * ip_text_addr( ip, buf, size ): converts the ip_addr_t pointed to by ip to
 * a string. If buf is 0 or size is less than the string length required,
 * uses a static buffer; otherwise, buf is used to store the string. 
 */
extern
char * ip_text_addr( register ip_addr_t *, register char *, register size_t);

/**
 * ip_text_addr( ip, buf, size ): converts the ip_addr_t ip to
 * a string. If buf is 0 or size is less than the string length required,
 * uses a static buffer; otherwise, buf is used to store the string. 
 */
extern
char * ip_text(ip_addr_t ip, register char *, register size_t);

/**
 * ip_addr_binary ( buf, len ): returns the ip_addr_t representation
 * of the len address bytes pointed to by buf; if len is not a 
 * recognized address length (the fixed length of an AF_INET, AF_INET6,
 * or AF_LLC ethernet address), returns a null ip address with sa_family==0.
 */
extern
ip_addr_t ip_addr_binary(uint8_t *buf, uint8_t len);

/**
 * ip_v4_broadcast( ip, prefix ): returns the IPv4 broadcast
 * address of the AF_INET IPv4 address pointed to by ip with
 * the given prefix .
 */
extern
ip_addr_t ip_v4_broadcast( ip_addr_t *, uint8_t prefix );

/**
 * ip_v4_netmask_to_prefix( netmask ): returns the prefix length
 * of all-one bits in the ip_addr_t pointed to by netmask. 
 */
extern
uint8_t ip_v4_netmask_to_prefix( ip_addr_t *netmask );

/**
 * ip_v4_prefix_to_netmask( prefix ): returns the netmask address
 * of prefix all-one bits .
 */
extern
ip_addr_t ip_v4_prefix_to_netmask( uint8_t prefix );

/**
 * ip_mask( ip, prefix ): returns the subnet address of the IPv4 or IPv6
 * address pointed to by ip with the given prefix.
 */
extern
ip_addr_t ip_mask( ip_addr_t *, uint8_t prefix );

/**@}*/
#endif
